// Create instance of Ogma
var ogma = new Ogma({
  container: 'graph-container',
  renderer: 'canvas'
});

var ANIMATION_DURATION = 1000;

// Load the graph and apply a layout
ogma.parse.jsonFromUrl('iphone_parts.json').then(function(graph) {
  return ogma.setGraph(graph);
}).then(function() {
  return runLayout();
});

// Style rule for basic nodes
ogma.styles.addRule({
  nodeAttributes: {
    icon: {
      font: 'FontAwesome',
      content: ogma.rules.map({
        field: 'type',
        values: {
          'part': '\uf013',
          'manufacturer': '\uf1ad',
          'device': '\uf10b'
        }
      })
    },
    color: ogma.rules.map({
      field: 'type',
      values: {
        'part': 'orange',
        'manufacturer': 'limegreen',
        'device': 'orangered'
      }
    }),
    text: {
      content: function(node) {
        var type = node.getData('type');

        if (type === 'device') {
          return node.getData('name');
        } else if (type === 'part') {
          return node.getData('part_type');
        } else if (type === 'manufacturer') {
          return node.getData('name');
        }
      }
    },
    image: {
      url: function(node) {
        if (node.getData('type') === 'country') {
          return '../flags/' + node.getData('iso') + '.svg';
        }
      }
    }
  }
});

// Style rules for generated neighbors
ogma.styles.addRule({
  nodeSelector: function(node) {
    return node.getData('type') === 'country';
  },
  nodeAttributes: {
    image: function(node) {
      return '../flags/' + node.getData('iso') + '.svg';
    },
    text: function(node) {
      return node.getData('iso');
    },
    radius: function(node) {
      return 5 + node.getData('nb_parts_produced');
    },
    innerStroke: {
      color: 'black'
    }
  }
});

// Enable the legend
ogma.tools.legend.enable({
  titleFontSize: 14,
  fontSize: 12,
  titleTextAlign: 'center',
  titleFunction: function(propertyPath) {
    var str = propertyPath.toString();
    return str.charAt(0).toUpperCase() + str.substring(1);
  }
});

// Add neighbor generation - creates the country nodes from the `country` data property of the manufacturer nodes
var neighborGeneration = ogma.transformations.addNeighborGeneration({
  selector: function(node) {
    return node.getData('type') === 'manufacturer';
  },
  neighborIdFunction: function(node) {
    return node.getData('country');
  },
  nodeGenerator: function(id, nodes) {
    return {
      data: {
        type: 'country',
        iso: id,
        nb_parts_produced: nodes.size
      }
    }
  },
  // The transformation is added disabled, so it can be toggled later
  enabled: false
});

// Add node collapsing - hides the manufacturer nodes between a country and a part, and replace them with an edge
var nodeCollapsing = ogma.transformations.addNodeCollapsing({
  selector: function(node) {
    return node.getData('type') === 'manufacturer';
  },
  edgeGenerator: function(hiddenNode) {
    return {
      attributes: {
        text: hiddenNode.getData('name')
      }
    }
  },
  // The transformation is added disabled, so it can be toggled later
  enabled: false
});

var neighborGenerationToggleButton = document.getElementById('neighbor-generation-toggle-button');
var nodeCollapsingToggleButton = document.getElementById('node-collapsing-toggle-button');
var runLayoutButton = document.getElementById('layout-button');

function disableAllButtons() {
  neighborGenerationToggleButton.disabled = true;
  nodeCollapsingToggleButton.disabled = true;
  runLayoutButton.disabled = true;
}

function enableAllButtons() {
  neighborGenerationToggleButton.disabled = false;
  nodeCollapsingToggleButton.disabled = false;
  runLayoutButton.disabled = false;
}

function toggleNeighborGeneration() {
  var enabled =  neighborGeneration.isEnabled();
  var promise = enabled ? neighborGeneration.disable(ANIMATION_DURATION) : neighborGeneration.enable(ANIMATION_DURATION);

  if (!enabled) {
    // Neighbors are generated at the center of the nodes, so we need to re-apply the layout
    promise = promise.then(function() {
      ogma.layouts.forceLink({locate: true});
    });
  }

  var newText = (enabled ? 'Show' : 'Hide') + ' countries';
  disableAllButtons();

  promise.then(function() {
    neighborGenerationToggleButton.textContent = newText;
    neighborGenerationToggleButton.disabled = false;
    runLayoutButton.disabled = false;

    if (!enabled) {
      nodeCollapsingToggleButton.disabled = false;
    }
  });
}

function toggleNodeCollapsing() {
  var enabled =  nodeCollapsing.isEnabled();
  var promise = enabled ? nodeCollapsing.disable(ANIMATION_DURATION) : nodeCollapsing.enable(ANIMATION_DURATION);

  var newText = (enabled ? 'Collapse' : 'Show') + ' manufacturers';
  disableAllButtons();

  promise.then(function() {
    nodeCollapsingToggleButton.textContent = newText;
    nodeCollapsingToggleButton.disabled = false;
    runLayoutButton.disabled = false;

    if (enabled) {
      neighborGenerationToggleButton.disabled = false;
    }
  });
}

function runLayout() {
  var neighborGenerationButtonDisabled = neighborGenerationToggleButton.disabled;
  var nodeCollapsingButtonDisabled = nodeCollapsingToggleButton.disabled;

  disableAllButtons();

  return ogma.layouts.forceLink({locate: true}).then(function() {
    runLayoutButton.disabled = false;

    if (!neighborGenerationButtonDisabled) {
      neighborGenerationToggleButton.disabled = false;
    }

    if (!nodeCollapsingButtonDisabled.disabled) {
      nodeCollapsingToggleButton.disabled = false;
    }
  });
}
